package ca.uwaterloo.cs.jgrok.lib;

import ca.uwaterloo.cs.jgrok.env.Env;
import ca.uwaterloo.cs.jgrok.fb.*;
import ca.uwaterloo.cs.jgrok.interp.*;

/**
 * Embed edges into a tree.
 *
 * <pre>
 *     EdgeSet edist(EdgeSet edges)
 *     EdgeSet edist(EdgeSet edges, EdgeSet contain)
 * </pre>
 */
public class EDistance extends Function {
    
    public EDistance() {
        name = "edist";
        type = EdgeSet.class;
        paramTypes = new Class[1];
        paramTypes[0] = EdgeSet.class;
    }
    
    public Value invoke(Env env, Value[] vals)
        throws InvocationException {
        EdgeSet edges;
        EdgeSet contain;
        
        edges = (EdgeSet)vals[0].objectValue();
        
        if(vals.length == 2) {
            contain = (EdgeSet)vals[1].objectValue();
        } else {
            try {
                Variable var;
                var = env.peepScope().lookup("contain");
                if(var.getType() != EdgeSet.class) {
                    throw new InvocationException("contain is not relation");
                }
                contain = (EdgeSet)var.getValue().objectValue();
            } catch(Exception e) {
                throw new InvocationException("contain not found");
            }
        }
        
        EdgeTree tree = new EdgeTree(contain);
        EdgeSet edgeDistance = tree.getEdgeDistance(edges);
        return new Value(edgeDistance);
    }
}
