package ca.uwaterloo.cs.jgrok.lib;

import ca.uwaterloo.cs.jgrok.env.Env;
import ca.uwaterloo.cs.jgrok.fb.Node;
import ca.uwaterloo.cs.jgrok.fb.NodeSet;
import ca.uwaterloo.cs.jgrok.interp.Value;

import java.util.regex.PatternSyntaxException;

/**
 * <pre>
 *   set grep(set, regex)
 * </pre>
 */
public class Grep extends Function {
    
    public Grep() {
        name = "grep";
        
        // default function
        type = NodeSet.class;
        paramTypes = new Class[2];
        paramTypes[0] = NodeSet.class;
        paramTypes[1] = String.class;
    }
    
    public Value invoke(Env env, Value[] vals)
        throws InvocationException {
        NodeSet result = new NodeSet();
        NodeSet set = (NodeSet)vals[0].objectValue();
        String regex = vals[1].toString();
        
        int len = regex.length();
        if(!((regex.charAt(0) == '^') ||
             (len > 1 && regex.substring(0, 2).equals(".*")))) {
            regex = ".*" + regex;
        }
        
        len = regex.length();
        if(!((regex.charAt(len-1) == '$') ||
             (len > 1 && regex.substring(len-2, len).equals(".*")))) {
            regex = regex + ".*";
        }
        
        String s;
        Node node;
        Node[] nodes = set.getAllNodes();
        
        for(int i = 0; i < nodes.length; i++) {
            node = nodes[i];
            s = node.get();
            try {
                if(s.matches(regex)) {
                    result.add(node.getID());
                } else if(s.indexOf(regex) > 0) {
                    result.add(node.getID());
                }
            } catch(PatternSyntaxException e) {
                throw new InvocationException(e.getMessage());
            }
        }
        
        return new Value(result);
    }
}
