package ca.uwaterloo.cs.jgrok.lib;

import ca.uwaterloo.cs.jgrok.env.Env;
import ca.uwaterloo.cs.jgrok.fb.*;
import ca.uwaterloo.cs.jgrok.interp.Value;

/**
 * <pre>
 *     TupleSet reverse(TupleSet tuples)
 * </pre>
 * 
 * Note: function <b>reverse</b> should be used in combination of function <b>sort</b> and <b>sortd</b>.
 * The following must hold true.
 * <pre>
 *     reverse(sort(nodes)) == sortd(nodes)
 *     reverse(sortd(nodes)) == sort(nodes)
 * </pre>
 */
public class Reverse extends Function {
    
    public Reverse() {
        name = "reverse";
        type = TupleSet.class;
        paramTypes = new Class[1];
        paramTypes[0] = TupleSet.class;
    }
    
    public Value invoke(Env env, Value[] vals)
        throws InvocationException { 
        TupleList tList;
        TupleSet input, output;
        
        input = (TupleSet)vals[0].objectValue();
        tList = input.getTupleList();
        output = input.newSet();
        
        int index;
        int count = tList.size();
        for(int i = 1; i <= count; i++) {
            index = count - i;
            output.add((Tuple)tList.get(index).clone());
        }
        
        return new Value(output);
    }
}
