package ca.uwaterloo.cs.jgrok.lib.math;

import java.util.Arrays;
import java.util.HashMap;

import ca.uwaterloo.cs.jgrok.env.Env;
import ca.uwaterloo.cs.jgrok.fb.*;
import ca.uwaterloo.cs.jgrok.interp.*;
import ca.uwaterloo.cs.jgrok.lib.Function;
import ca.uwaterloo.cs.jgrok.lib.InvocationException;

/**
 * <pre>
 * Functions:
 *
 *     set min(set)
 *     set min(set, int)
 *
 *     int mini(set)
 *     float minf(set)
 *
 * </pre>
 */
public class Min extends Function {
    private static final int METHOD_MIN = 0;
    private static final int METHOD_MIN_INT = 1;
    private static final int mETHOD_MIN_FLOAT = 2;
    private int whichMethod = METHOD_MIN;
    
    public Min(String functionName) {
        this.name = functionName;
        if(this.name.equals("min")) {
            whichMethod = METHOD_MIN;
        } else if(this.name.equals("mini")) {
            whichMethod = METHOD_MIN_INT;
        } else if(this.name.equals("minf")) {
            whichMethod = mETHOD_MIN_FLOAT;
        }
        
        type = NodeSet.class;
        paramTypes = new Class[1];
        paramTypes[0] = NodeSet.class;
    }
    
    public Value invoke(Env env, Value[] vals)
        throws InvocationException {
        if(vals.length == 2) {
            return multiset(vals);
        }
        
        float minFloat = 0;
        NodeSet result = null;
        NodeSet set = (NodeSet)vals[0].objectValue();
            
        Node node;
        Node[] data;
        data = set.getAllNodes();
        if(data.length > 0) {
            int i = 0;
            float val = -1;
            for(; i < data.length; i++) {
                node = data[i];
                try {
                    val = Float.parseFloat(node.get());
                    minFloat = val;
                    break;
                } catch(NumberFormatException e) {}
            }
            
            if(i < data.length) {
                String s;
                String s_min = data[i].get();
                
                for(; i < data.length; i++) {
                    node = data[i];
                    try {
                        s = node.get();
                        val = Float.parseFloat(s);
                        if(val < minFloat){
                            s_min = s;
                            minFloat = val;
                        }
                    } catch(NumberFormatException e) {}
                }
                result = NodeSet.singleton(s_min);
            }
        }
        
        switch(whichMethod) {
        case METHOD_MIN:
            if(result == null) result = new NodeSet();
            return new Value(result);
        case METHOD_MIN_INT:
            if(result == null) break;
            return new Value((int)minFloat);
        case mETHOD_MIN_FLOAT:
            if(result == null) break;
            return new Value(minFloat);
        }
        
        return Value.VOID;
    }
    
    private Value multiset(Value[] vals) {
        int count;
        NodeSet set;
        NodeSet result;
        
        set = (NodeSet)vals[0].objectValue();
        count = vals[1].intValue();
        result = new NodeSet();
        
        if(count > 0) {
            Node[] data;
            HashMap<Float,String> map;
            
            data = set.getAllNodes();            
            map = new HashMap<Float,String>(101, 0.75f); 
            
            Float f;
            String s;
            for(int i = 0; i < data.length; i++) {
                try {
                    s = data[i].get();
                    f = new Float(s);
                    map.put(f, s);
                } catch(NumberFormatException e) {}
            }
            
            Object[] keys;
            keys = map.keySet().toArray();
            Arrays.sort(keys);
            
            for(int i = 0; i < keys.length && count > 0; i++, count--) {
                result.add((String)map.get(keys[i]));
            }
        }
        
        return new Value(result);
    }
}
