package ca.uwaterloo.cs.jgrok.lib.math;

import ca.uwaterloo.cs.jgrok.env.Env;
import ca.uwaterloo.cs.jgrok.fb.*;
import ca.uwaterloo.cs.jgrok.interp.*;
import ca.uwaterloo.cs.jgrok.lib.Function;
import ca.uwaterloo.cs.jgrok.lib.InvocationException;

/**
 * <pre>
 *     float stdev(NodeSet)
 *     float stdev(TupleSet, col)
 * </pre>
 */
public class Stdev extends Function {
    
    public Stdev() {
        name = "stdev";
        type = float.class;
        paramTypes = new Class[1];
        paramTypes[0] = NodeSet.class;
    }
    
    public Value invoke(Env env, Value[] vals)
        throws InvocationException {
        float sum;
        float mean;
        float result;
        TupleSet input;
        
        sum = 0;
        result = 0;
        input = (TupleSet)vals[0].objectValue();
        
        if(input.size() > 0) {
            TupleList data = input.getTupleList();
            
            if(vals.length == 1) {
                for(int i = 0; i < data.size(); i++) {
                    Tuple tuple = data.get(i);
                    try {
                        sum += Float.parseFloat(IDManager.get(tuple.getDom()));
                    } catch(NumberFormatException e) {}
                }
            } else {
                int col = vals[1].intValue();
                for(int i = 0; i < data.size(); i++) {
                    Tuple tuple = data.get(i);
                    if(col < tuple.size()) {
                        try {
                            sum += Float.parseFloat(IDManager.get(tuple.get(col)));
                        } catch(NumberFormatException e) {}
                    } else {
                        throw new InvocationException("index " + col + " out of bounds " + tuple.size());
                    }
                }
            }
            
            mean = sum / input.size();
            sum = 0;
            
            float f;
            if(vals.length == 1) {
                for(int i = 0; i < data.size(); i++) {
                    Tuple tuple = data.get(i);
                    try {
                        f = Float.parseFloat(IDManager.get(tuple.getDom()));
                        sum += (f-mean)*(f-mean);
                    } catch(NumberFormatException e) {}
                }
            } else {
                int col = vals[1].intValue();
                for(int i = 0; i < data.size(); i++) {
                    Tuple tuple = data.get(i);
                    if(col < tuple.size()) {
                        try {
                            f = Float.parseFloat(IDManager.get(tuple.get(col)));
                            sum += (f-mean)*(f-mean);
                        } catch(NumberFormatException e) {}
                    } else {
                        throw new InvocationException("index " + col + " out of bounds " + tuple.size());
                    }
                }
            }
            
            if(input.size() > 1) {
                result = sum / (input.size() -  1);
                result = (float)Math.sqrt(result);
            } else result = 0;
        }
        
        return new Value(result);
    }
}
